package com.sikadi.user.service.impl;

import cn.hutool.core.util.StrUtil;
import com.sikadi.user.dto.MsgDTO;
import com.sikadi.user.listener.XfXhStreamClient;
import com.sikadi.user.listener.XfXhWebSocketListener;
import com.sikadi.user.mapper.XfXhConfig;
import com.sikadi.user.service.GTPService;
import lombok.extern.log4j.Log4j2;
import okhttp3.WebSocket;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.UUID;


/**
 * 修改
 **/
@Service
@Log4j2
public class GTPServiceImpl implements GTPService {

    @Autowired
    private XfXhStreamClient xfXhStreamClient;

    @Autowired
    private XfXhConfig xfXhConfig;

    @Override
    public String send(String ques) {

        // 如果是无效字符串，则不对大模型进行请求
        if (StrUtil.isBlank(ques)) {
            return "无效问题，请重新输入";
        }

        // 创建消息对象
        MsgDTO msgDTO = MsgDTO.createUserMsg(ques);
        // 创建监听器
        XfXhWebSocketListener listener = new XfXhWebSocketListener();
        // 发送问题给大模型，生成 websocket 连接
        WebSocket webSocket = xfXhStreamClient.sendMsg(UUID.randomUUID().toString().substring(0, 10), Collections.singletonList(msgDTO), listener);
        if (webSocket == null) {
            // 归还令牌
            xfXhStreamClient.operateToken(XfXhStreamClient.BACK_TOKEN_STATUS);
            return "系统内部错误，请联系管理员";
        }
        try {
            int count = 0;
            // 为了避免死循环，设置循环次数来定义超时时长
            int maxCount = xfXhConfig.getMaxResponseTime() * 5;
            while (count <= maxCount) {
                Thread.sleep(200);
                if (listener.isWsCloseFlag()) {
                    break;
                }
                count++;
            }
            if (count > maxCount) {
                return "大模型响应超时，请联系管理员";
            }
            // 响应大模型的答案
            return listener.getAnswer().toString();
        } catch (InterruptedException e) {
            log.error("错误：" + e.getMessage());
            return "系统内部错误，请联系管理员";
        } finally {
            // 关闭 websocket 连接
            webSocket.close(1000, "");
            // 归还令牌
            xfXhStreamClient.operateToken(XfXhStreamClient.BACK_TOKEN_STATUS);
        }
    }
}
